<?php

namespace Drupal\commerce_webpay_by\Controller;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class WebpayByRedirectController.
 */
class WebpayByRedirectController extends ControllerBase {

  /**
   * @param array $data
   *
   * @return string
   */
  public function signature(array $data) {
    $signature = '';

    /**
     * @var string $batch_timestamp
     * @var string $currency_id
     * @var string $amount
     * @var string $payment_method
     * @var string $order_id
     * @var string $site_order_id
     * @var string $transaction_id
     * @var string $payment_type
     * @var string $rrn
     * @var string $secret_key
     */
    extract($data);

    $signature .= $batch_timestamp;
    $signature .= $currency_id;
    $signature .= $amount;
    $signature .= $payment_method;
    $signature .= $order_id;
    $signature .= $site_order_id;
    $signature .= $transaction_id;
    $signature .= $payment_type;
    $signature .= $rrn;
    $signature .= $secret_key;

    return md5($signature);
  }


  /**
   * Notify page callback.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *
   * @return Response|array
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function notify(Request $request) {
    $data = $request->request->all();

    if ($data['site_order_id']) {
      $order = $this->getOrder($data['site_order_id']);
      /** @var \Drupal\commerce_payment\Entity\PaymentGateway $payment_gateway */
      $payment_gateway = $order->get('payment_gateway')->first()->entity;
      $data['secret_key'] = $payment_gateway->get('configuration')['secret_key'] ?? '';
      $signature = $this->signature($data);

      if ($signature === $data['wsb_signature']) {
        try {
          $this->completePayment($order, $data);
          return new Response();
        } catch (\Exception $e) {
          $message = $this->t('Error during creating new payment: @exception', [
            '@exception' => $e->getMessage(),
          ]);
          \Drupal::logger('commerce_webpay_by')->emergency($message, [
            'link' => $order->toLink('Order')->toString(),
          ]);
        }
      }
      else {
        $message = $this->t('Signature of payment was wrong!');
        \Drupal::logger('commerce_webpay_by')->error($message, [
          'link' => $order->toLink('Order')->toString(),
        ]);
      }
    }
    else {
      $message = $this->t('No order id defined in the API request!');
      \Drupal::logger('commerce_webpay_by')->emergency($message);
    }

    return new Response('', 403);
  }


  /**
   * Get order by ID.
   *
   * @param int $order_id
   *
   * @return \Drupal\commerce_order\Entity\OrderInterface
   */
  public function getOrder(int $order_id) {
    return \Drupal\commerce_order\Entity\Order::load($order_id);
  }


  /**
   * Complete new payment.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   * @param array                                        $data
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityMalformedException
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  public function completePayment(OrderInterface $order, $data) {
    /** @var \Drupal\commerce_payment\Entity\PaymentGateway $payment_gateway */
    $payment_gateway = $order->get('payment_gateway')->first()->entity;
    $payment_storage = \Drupal::entityTypeManager()->getStorage('commerce_payment');
    $payment = $payment_storage->create([
      'state'           => 'completed',
      'amount'          => $order->getTotalPrice(),
      'payment_gateway' => $payment_gateway->id(),
      'order_id'        => $order->id(),
      'remote_id'       => $data['order_id'],
      'remote_state'    => $data['payment_method'],
    ]);
    $payment->save();

    $message = $this->t('New payment for order #@order', ['@order' => $order->id()]);
    \Drupal::logger('commerce_webpay_by')->info($message, [
      'link' => $order->toLink('Order')->toString(),
    ]);
  }
}
